@@ -4,6 +4,7 @@ from snowflake.snowpark import Session
 from snowflake.snowpark.dataframe import DataFrame
 import pandas as pd
 import numpy as np
+from . import table_creator
 
 logger = logging.getLogger(__name__)
 
@@ -94,7 +95,9 @@ def write_dataframe_to_table(
     validate_types: bool = True,
     use_logical_type: bool = True,
     on_error: str = "CONTINUE",
-    df_fields: Optional[dict] = None
+    df_fields: Optional[dict] = None,
+    snowflake_fields: Optional[dict] = None,
+    force_full_sync: bool = False
 ) -> bool:
     """
     Centralized function for writing DataFrames to Snowflake tables.
@@ -123,9 +126,15 @@ def write_dataframe_to_table(
         if df_fields is not None:
             try:
                 from lht.util import field_types
-                logger.info(f"🔧 Formatting DataFrame using field_types.format_sync_file with {len(df_fields)} field definitions")
                 df = field_types.format_sync_file(df, df_fields)
-                logger.info(f"✅ DataFrame formatting completed")
+                
+                # VALIDATION: Ensure snowflake_fields is provided when df_fields is available
+                if snowflake_fields is None or len(snowflake_fields) == 0:
+                    logger.error(f"❌ CRITICAL: df_fields provided but snowflake_fields is missing!")
+                    logger.error(f"   This means sobjects.describe() failed or wasn't called")
+                    logger.error(f"   Cannot proceed without proper Salesforce field type definitions")
+                    raise Exception(f"Missing snowflake_fields when df_fields is provided for {schema}.{table}")
+                    
             except Exception as e:
                 logger.warning(f"⚠️ Failed to format DataFrame with field_types: {e}")
                 logger.warning(f"   Continuing with original DataFrame")
@@ -164,16 +173,115 @@ def write_dataframe_to_table(
         logger.debug(f"   - Use logical types: {use_logical_type}")
         logger.debug(f"   - On error: {on_error}")
         
-        # Write DataFrame to table
-        result = session.write_pandas(
-            df, 
-            full_table_name,
-            quote_identifiers=False, 
-            auto_create_table=auto_create,
-            overwrite=overwrite, 
-            use_logical_type=use_logical_type, 
-            on_error=on_error
-        )
+        # If we have field definitions and need to create a table, use centralized table creation
+        if auto_create and df_fields is not None and len(df_fields) > 0:
+            try:
+                # The DataFrame has already been processed by field_types.format_sync_file above
+                # No need to process it again - use the already-processed DataFrame
+                
+                # Use provided snowflake_fields if available, otherwise infer from DataFrame
+                if snowflake_fields is not None and len(snowflake_fields) > 0:
+                    # Use the Salesforce field types directly (correct approach)
+                    pass
+                else:
+                    # No Salesforce field types provided - this should not happen in normal operation
+                    logger.error(f"❌ CRITICAL: No Salesforce field types provided for table creation!")
+                    logger.error(f"   This means sobjects.describe() failed or wasn't called")
+                    logger.error(f"   Cannot create table without proper field type definitions")
+                    raise Exception(f"No Salesforce field types provided for table {schema}.{table}")
+                
+
+                # Use centralized table creation
+                print(f"🔍 About to call table_creator.ensure_table_exists_for_dataframe with force_full_sync={force_full_sync}")
+                table_creator.ensure_table_exists_for_dataframe(
+                    session=session,
+                    schema=schema,
+                    table=target_table,
+                    df_fields=df_fields,
+                    snowflake_fields=snowflake_fields,
+                    force_full_sync=force_full_sync,  # Pass through the force_full_sync parameter
+                    database=current_db
+                )
+                print(f"✅ Table creation completed, now writing data with overwrite={overwrite}")
+                
+                # Now write to the existing table using the already-processed DataFrame
+                # Convert all datetime fields to timezone-naive for Snowflake compatibility
+                for col in df.columns:
+                    if col.upper() in ['CREATEDDATE', 'LASTMODIFIEDDATE', 'SYSTEMMODSTAMP', 'LASTACTIVITYDATE', 'LASTVIEWEDDATE', 'LASTREFERENCEDDATE']:
+                        if 'UTC' in str(df[col].dtype) or 'timezone' in str(df[col].dtype):
+                            try:
+                                df[col] = df[col].dt.tz_localize(None)
+                            except Exception as e:
+                                logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
+                
+                #df['CREATEDDATE'] = pd.to_datetime(df['CREATEDDATE'], errors='coerce')
+                # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
+                print(f"🔍 Writing data with mode: {'overwrite' if overwrite else 'append'}")
+                print(f"🔍 Target table: {full_table_name}")
+                snowpark_df = session.create_dataframe(df)
+                snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
+                print(f"✅ Data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+                result = True  # save_as_table doesn't return a result object like write_pandas
+            except Exception as table_error:
+                logger.warning(f"Centralized table creation failed, falling back to auto-create: {table_error}")
+                # CRITICAL FIX: Use the processed DataFrame for fallback, not the original
+                logger.info(f"🔧 Using processed DataFrame for fallback write...")
+                try:
+                    from lht.util import field_types
+                    df_fallback = field_types.format_sync_file(df, df_fields)
+                    logger.info(f"✅ DataFrame processing completed for fallback write")
+                except Exception as e:
+                    logger.warning(f"⚠️ Failed to process DataFrame with field_types for fallback: {e}")
+                    df_fallback = df
+                
+                # TEST: Keeping all datetime fields for debugging (fallback path)
+                logger.info(f"🧪 TEST: Keeping all datetime fields to debug the issue (fallback path)")
+                logger.info(f"✅ TEST: All columns preserved in fallback: {list(df_fallback.columns)}")
+                
+                # Convert all datetime fields to timezone-naive for Snowflake compatibility
+                # for col in df_fallback.columns:
+                #     if col.upper() in ['CREATEDDATE', 'LASTMODIFIEDDATE', 'SYSTEMMODSTAMP', 'LASTACTIVITYDATE', 'LASTVIEWEDDATE', 'LASTREFERENCEDDATE']:
+                #         if 'UTC' in str(df_fallback[col].dtype) or 'timezone' in str(df_fallback[col].dtype):
+                #             try:
+                #                 df_fallback[col] = df_fallback[col].dt.tz_localize(None)
+                #             except Exception as e:
+                #                 logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
+                
+                # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
+                print(f"🔍 Writing fallback data with mode: {'overwrite' if overwrite else 'append'}")
+                print(f"🔍 Target table: {full_table_name}")
+                snowpark_df = session.create_dataframe(df_fallback)
+                snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
+                print(f"✅ Fallback data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+                result = True  # save_as_table doesn't return a result object like write_pandas
+        else:
+            # Use original behavior, but ensure DataFrame is properly processed
+            if df_fields is not None and len(df_fields) > 0:
+                try:
+                    from lht.util import field_types
+                    df_processed = field_types.format_sync_file(df, df_fields)
+                except Exception as e:
+                    logger.warning(f"⚠️ Failed to process DataFrame with field_types: {e}")
+                    df_processed = df
+            else:
+                df_processed = df
+            
+            # Convert all datetime fields to timezone-naive for Snowflake compatibility
+            for col in df_processed.columns:
+                if col.upper() in ['CREATEDDATE', 'LASTMODIFIEDDATE', 'SYSTEMMODSTAMP', 'LASTACTIVITYDATE', 'LASTVIEWEDDATE', 'LASTREFERENCEDDATE']:
+                    if 'UTC' in str(df_processed[col].dtype) or 'timezone' in str(df_processed[col].dtype):
+                        try:
+                            df_processed[col] = df_processed[col].dt.tz_localize(None)
+                        except Exception as e:
+                            logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
+            
+            # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
+            print(f"🔍 Writing processed data with mode: {'overwrite' if overwrite else 'append'}")
+            print(f"🔍 Target table: {full_table_name}")
+            snowpark_df = session.create_dataframe(df_processed)
+            snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
+            print(f"✅ Processed data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+            result = True  # save_as_table doesn't return a result object like write_pandas
         
         logger.debug(f"✅ Successfully wrote {len(df)} records to {full_table_name}")
         return True
@@ -190,6 +298,9 @@ def write_dataframe_to_table(
         
         raise Exception(f"DataFrame write operation failed: {e}")
 
+
+
+
 def standardize_dataframe_types(df: pd.DataFrame, type_strategy: str = "auto") -> pd.DataFrame:
     """
     Standardize DataFrame data types to prevent Snowflake casting issues.
@@ -212,16 +323,13 @@ def standardize_dataframe_types(df: pd.DataFrame, type_strategy: str = "auto") -
         if type_strategy == "string":
             # Convert everything to string
             df_standardized[column] = col_data.astype(str)
-            logger.debug(f"   Column '{column}': {original_dtype} → string")
             
         elif type_strategy == "numeric":
             # Try to convert to numeric where possible, string otherwise
             try:
                 df_standardized[column] = pd.to_numeric(col_data, errors='coerce')
-                logger.debug(f"   Column '{column}': {original_dtype} → numeric")
             except (ValueError, TypeError):
                 df_standardized[column] = col_data.astype(str)
-                logger.debug(f"   Column '{column}': {original_dtype} → string (numeric conversion failed)")
                 
         elif type_strategy == "lenient":
             # More intelligent type conversion
@@ -247,13 +355,10 @@ def standardize_dataframe_types(df: pd.DataFrame, type_strategy: str = "auto") -
                     if numeric_count / total_count > 0.8:
                         try:
                             df_standardized[column] = pd.to_numeric(col_data, errors='coerce')
-                            logger.debug(f"   Column '{column}': {original_dtype} → numeric (80%+ numeric values)")
                         except (ValueError, TypeError):
                             df_standardized[column] = col_data.astype(str)
-                            logger.debug(f"   Column '{column}': {original_dtype} → string (numeric conversion failed)")
                     else:
                         df_standardized[column] = col_data.astype(str)
-                        logger.debug(f"   Column '{column}': {original_dtype} → string (mixed types)")
                         
         else:  # "auto" strategy
             # Smart type inference
@@ -278,17 +383,13 @@ def standardize_dataframe_types(df: pd.DataFrame, type_strategy: str = "auto") -
                     if all_numeric:
                         try:
                             df_standardized[column] = pd.to_numeric(col_data, errors='coerce')
-                            logger.debug(f"   Column '{column}': {original_dtype} → numeric (all values numeric)")
                         except (ValueError, TypeError):
                             df_standardized[column] = col_data.astype(str)
-                            logger.debug(f"   Column '{column}': {original_dtype} → string (numeric conversion failed)")
                     elif all_strings:
                         df_standardized[column] = col_data.astype(str)
-                        logger.debug(f"   Column '{column}': {original_dtype} → string (all values strings)")
                     else:
                         # Mixed types - convert to string to be safe
                         df_standardized[column] = col_data.astype(str)
-                        logger.debug(f"   Column '{column}': {original_dtype} → string (mixed types detected)")
     
     logger.info(f"✅ DataFrame type standardization completed")
     return df_standardized
@@ -343,7 +444,8 @@ def write_batch_to_temp_table(
     schema: str,
     temp_table: str,
     df_fields: dict,
-    validate_types: bool = True
+    validate_types: bool = True,
+    force_full_sync: bool = False
 ) -> bool:
     """
     Write a batch DataFrame to a temporary table for incremental sync operations.
@@ -373,7 +475,8 @@ def write_batch_to_temp_table(
             validate_types=validate_types,
             use_logical_type=False,  # More lenient for temp tables
             on_error="CONTINUE",
-            df_fields=df_fields  # Pass field definitions for proper formatting
+            df_fields=df_fields,  # Pass field definitions for proper formatting
+            force_full_sync=force_full_sync  # Pass through force_full_sync parameter
         )
     except Exception as e:
         error_msg = str(e)
@@ -394,7 +497,8 @@ def write_batch_to_temp_table(
                 validate_types=False,
                 use_logical_type=False,
                 on_error="CONTINUE",
-                df_fields=df_fields  # Pass field definitions for proper formatting
+                df_fields=df_fields,  # Pass field definitions for proper formatting
+                force_full_sync=force_full_sync  # Pass through force_full_sync parameter
             )
         else:
             logger.error(f"❌ Non-casting error in temp table write: {error_msg}")
@@ -408,7 +512,9 @@ def write_batch_to_main_table(
     is_first_batch: bool = False,
     validate_types: bool = True,
     use_logical_type: bool = True,
-    df_fields: Optional[dict] = None
+    df_fields: Optional[dict] = None,
+    snowflake_fields: Optional[dict] = None,
+    force_full_sync: bool = False
 ) -> bool:
     """
     Write a batch DataFrame to the main table (overwrite for first batch, append for subsequent).
@@ -425,10 +531,21 @@ def write_batch_to_main_table(
     Returns:
         bool: True if successful
     """
-    # CRITICAL FIX: Never use overwrite=True for first batch
-    # This prevents write_pandas from inferring wrong column types
-    overwrite = False  # Always append, never overwrite
-    logger.debug(f"💾 Writing batch to main table: {schema}.{table} (overwrite={overwrite}, is_first_batch={is_first_batch})")
+    # Handle overwrite logic based on force_full_sync and batch type
+    if force_full_sync and is_first_batch:
+        # Force full sync: drop and recreate table, then overwrite first batch
+        overwrite = True
+        print(f"💾 Force full sync: overwriting first batch to recreate table")
+    elif is_first_batch:
+        # Regular first batch: append to existing or new table
+        overwrite = False
+        print(f"💾 Regular first batch: appending to table")
+    else:
+        # Subsequent batches: always append
+        overwrite = False
+        print(f"💾 Subsequent batch: appending to table")
+    
+    print(f"💾 Writing batch to main table: {schema}.{table} (overwrite={overwrite}, is_first_batch={is_first_batch}, force_full_sync={force_full_sync})")
     
     return write_dataframe_to_table(
         session=session,
@@ -440,5 +557,7 @@ def write_batch_to_main_table(
         validate_types=validate_types,
         use_logical_type=use_logical_type,
         on_error="CONTINUE",
-        df_fields=df_fields  # Pass field definitions for proper formatting
+        df_fields=df_fields,  # Pass field definitions for proper formatting
+        snowflake_fields=snowflake_fields,  # Pass Salesforce field types for proper table creation
+        force_full_sync=force_full_sync  # Pass through force_full_sync parameter
     )
